/* global Comment, Utils, WebKitPoint */

function FullScreenView(element, delegate) {
    this.element = element;
    this.delegate = delegate;

    this.photo = element.querySelector('#photo');
    this.commentsWrapper = element.querySelector('#comments-wrapper');
    this.comments = element.querySelector('#comments');
}

FullScreenView.prototype = {
    handleEvent: function (event) {
        this.element.removeEventListener('webkitTransitionEnd', this);

        if (this.transitionDirection === 1) {
            this.transitionInDidComplete();
        } else {
            this.transitionOutDidComplete();
        }
    },

    setData: function (data) {
        this.data = data;
        this.photo.src = 'images/full/' + data.src;
    },

    renderComments: function (comments) {
        this.comments.innerHTML = '';

        for (var i = 0, count = comments.length; i < count; i++) {
            var comment = new Comment();
            this.comments.appendChild(comment.element);

            comment.setInfo(comments[i]);
            comment.setIndex(i);
        }
    },

    transitionInFromTarget: function (item) {
        this.delegate.fullScreenTransitionInWillStart();
        this.transitionDirection = 1;

        // Suppress animations here by overriding CSS-defined styles.
        this.enableTransitions(false);
        this.positionElementRelativeToTarget(item);
        this.enableTransitions(true);

        this.element.addEventListener('webkitTransitionEnd', this);
        this.element.style.display = 'block';

        setTimeout(function () {
            this.element.style.opacity = 1;
            this.commentsWrapper.style.opacity = 1;
            this.positionFullScreen();
        }.bind(this), 0);
    },

    enableTransitions: function (enabled) {
        this.element.style.webkitTransition = enabled ? null : 'none';
        this.photo.style.webkitTransition = enabled ? null : 'none';
    },

    positionElementRelativeToTarget: function (item) {
        var point = this.getGlobalPositionForTarget(item);

        this.target = item;

        this.photo.style.webkitTransform = Utils.t3d(point.x, point.y) +
            'scale(' + (item.element.offsetWidth / window.innerWidth) + ')';
    },

    getGlobalPositionForTarget: function (item) {
        var relativePosition = item.position;
        var point = window.webkitConvertPointFromNodeToPage(
            item.element.parentNode,
            new WebKitPoint(relativePosition.x, relativePosition.y)
        );

        point.y -= window.scrollY;
        return point;
    },

    positionFullScreen: function () {
        this.photo.style.webkitTransform = 'translate3d(0, 0, 0) scale(1)';
    },

    transitionOutToTarget: function (target) {
        this.enableTransitions(true);
        this.delegate.fullScreenTransitionOutWillStart();
        this.transitionDirection = -1;
        this.element.addEventListener('webkitTransitionEnd', this);

        this.positionElementRelativeToTarget(target);
        this.element.style.opacity = 0;
        this.commentsWrapper.style.opacity = 0;
    },

    transitionInDidComplete: function () {
        this.enableTransitions(false);
        this.delegate.fullScreenTransitionInDidComplete();
        this.renderComments(this.data.comments || []);
    },

    transitionOutDidComplete: function () {
        this.element.style.display = 'none';
        this.delegate.fullScreenTransitionOutDidComplete();
    }
};
