/* global Photo */

var MIN_PHOTO_WIDTH = 200;
var PHOTOS_RATIO = 480 / 720;

function PhotoGrid(element, photosInfo, delegate) {
	this.element = element;
	this.photosInfo = photosInfo;
	this.delegate = delegate;

	// Initialize photos.
	this.photos = [];
	this.photosById = [];

	for (var i = 0, count = photosInfo.length; i < count; i++) {
		var photoInfo = photosInfo[i];
		// Assign id to the photo info.
		photoInfo.id = i;
		// Create the photo with the provided info and store it by its id.
		this.photosById[i] = this.initializePhoto(photosInfo[i]);
	}

	this.layoutPhotos();

	window.addEventListener('resize', this);
	this.element.addEventListener('webkitTransitionEnd', this);
}

PhotoGrid.prototype = {
	initializePhoto: function (photoInfo) {
		var photo = new Photo(photoInfo);

		this.photos.push(photo);
		this.element.appendChild(photo.element);

		photo.element.addEventListener('click', function (event) {

			// Notify the delegate that a photo was selected.
			this.notifyPhotoWasSelectedById(photoInfo.id);

		}.bind(this));

		return photo;
	},

	notifyPhotoWasSelectedById: function (id) {
		var photo = this.photosById[id];
		this.delegate.photoGridDidSelectPhoto(this, photo);
	},

	sortFunction: function () {
		return 0.5 - Math.random();
	},

	shufflePhotos: function () {
		//From the current layout get the current number of photos per row,
		//their resized width, and their resized height.
		var photosPerRow = Math.floor(document.body.offsetWidth / MIN_PHOTO_WIDTH);
		var photoWidth = Math.floor(document.body.offsetWidth / photosPerRow);
		var photoHeight = Math.floor(photoWidth * PHOTOS_RATIO);

		//Get the current offset from the top of the page to the start
		//of the photo section.
		var scrollY = Math.max(window.scrollY, 0);

		//Get the index of the first and last visible photo.
		var firstVisiblePhoto = Math.floor(scrollY / photoHeight) * photosPerRow;
		var lastVisiblePhoto = Math.ceil(window.innerHeight / photoHeight + 1) * photosPerRow + firstVisiblePhoto;

		var preSorted = this.photos.slice(0, firstVisiblePhoto);
		var postSorted = this.photos.slice(lastVisiblePhoto);
		var sorted = this.photos.slice(firstVisiblePhoto, lastVisiblePhoto).sort(this.sortFunction);

		this.photos = preSorted.concat(sorted).concat(postSorted);
		this.element.classList.add('shuffling');
		this.layoutPhotos();
	},

	layoutPhotos: function () {
		for (var i = 0, count = this.photos.length; i < count; i++) {
			var photo = this.photos[i];

			// Calculate dimensions of photos based on the current width of element.
			var currentWidth = this.element.offsetWidth;
			var photosPerRow = Math.floor(currentWidth / MIN_PHOTO_WIDTH);
			var photoWidth = Math.floor(currentWidth / photosPerRow);
			var photoHeight = Math.floor(photoWidth * PHOTOS_RATIO);

			// Set the photo's size.
			photo.setSize(photoWidth, photoHeight);

			// Translate photo to new position.
			photo.translate(
				Math.floor(photoWidth * (i % photosPerRow)),
				Math.floor(photoHeight * Math.floor(i / photosPerRow))
			);
		}

		this.element.style.height = Math.ceil(this.photos.length / photosPerRow) * photoHeight + 'px';
	},

	handleEvent: function (event) {
		if (event.type === 'resize') {
			this.layoutPhotos();
			return;
		}

		if (event.target.className === 'photo-wrapper') {
			this.element.classList.remove('shuffling');
			return;
		}
	}
};
